#!/usr/bin/env python

import os
import sys
import shutil
import tempfile
import inspect
import time

import numpy
import nose

import clawpack.geoclaw.dtopotools as dtopotools

# Set local test directory to get local files
testdir = os.path.dirname(__file__)
if len(testdir) == 0:
     testdir = "./"

def test_read_csv_make_dtopo(save=False):
    r"""Test reading and making of a CSV subfault speficied dtopo."""
    
    subfault_path = os.path.join(testdir, 'data', 'alaska1964.csv')
    input_units = {"length":"km", "width":"km", "depth":"km", "slip":"m", 
             "mu":"dyne/cm^2"}
    fault = dtopotools.CSVFault()
    fault.read(subfault_path, input_units=input_units, 
                              coordinate_specification="noaa sift")

    assert abs(fault.Mw() - 8.53336) < 1e-4, "*** Mw is wrong: %g" % fault.Mw()

    xlower = 203
    xupper = 214.  # approximate - adjusted below
    ylower = 55
    yupper = 60.  # approximate - adjusted below

    # dtopo parameters:
    points_per_degree = 4  # 15 minute resolution
    dx = 1. / points_per_degree
    mx = int((xupper - xlower) / dx + 1)
    xupper = xlower + (mx - 1) * dx
    my = int((yupper - ylower) / dx + 1)
    yupper = ylower + (my - 1) * dx

    x = numpy.linspace(xlower, xupper, mx)
    y = numpy.linspace(ylower, yupper, my)

    dtopo = fault.create_dtopography(x, y, times=[1.])

    test_data_path = os.path.join(testdir, "data", "alaska1964_test_data.tt3")
    if save:
        dtopo.write(test_data_path, dtopo_type=3)
    compare_data = dtopotools.DTopography(path=test_data_path)
    compare_data.read(path=test_data_path, dtopo_type=3)

    assert dtopo.dZ.shape == compare_data.dZ.shape, \
        "dtopo.dZ.shape is %s, should be %s" \
        % (dtopo.dZ.shape, compare_data.dZ.shape)

    assert numpy.allclose(compare_data.dZ, dtopo.dZ)


def test_read_ucsb_make_dtopo(save=False):
    r"""Test reading and making of a UCSB subfault speficied dtopo."""

    subfault_path = os.path.join(testdir, 'data', 'tohoku_ucsb.txt')
    fault = dtopotools.UCSBFault()
    fault.read(subfault_path)

    assert abs(fault.Mw() - 9.13957973) < 1e-4, "*** Mw is wrong: %g" % fault.Mw()

    xlower = 140.
    xupper = 146.
    ylower = 35.
    yupper = 41.

    # dtopo parameters:
    points_per_degree = 4  # 15 minute resolution
    dx = 1. / points_per_degree
    mx = int((xupper - xlower) / dx + 1)
    xupper = xlower + (mx - 1) * dx
    my = int((yupper - ylower) / dx + 1)
    yupper = ylower + (my - 1) * dx

    x = numpy.linspace(xlower, xupper, mx)
    y = numpy.linspace(ylower, yupper, my)

    tmax = 0.
    for s in fault.subfaults:
        tmax = max(tmax, s.rupture_time + s.rise_time)

    fault.rupture_type = 'kinematic'
    times = numpy.linspace(0, tmax, 10)
    dtopo = fault.create_dtopography(x, y, times)

    test_data_path = os.path.join(testdir, "data", "tohoku_test_data.tt3")
    if save:
        dtopo.write(test_data_path, dtopo_type=3)
    compare_data = dtopotools.DTopography(path=test_data_path)
    compare_data.read(path=test_data_path, dtopo_type=3)

    assert dtopo.dZ.shape == compare_data.dZ.shape, \
        "dtopo.dZ.shape is %s, should be %s" \
        % (dtopo.dZ.shape, compare_data.dZ.shape)

    assert numpy.allclose(compare_data.dZ, dtopo.dZ)


def test_read_sift_make_dtopo(save=False):
    r"""Test reading and making of a SIFT subfault speficied dtopo"""

    sift_slip = {'acsza1':2, 'acszb1':3}
    fault = dtopotools.SiftFault(sift_slip)

    assert abs(fault.Mw() - 7.966666666667) < 1e-4, "*** Mw is wrong: %g" % fault.Mw()

    xlower = 162.
    xupper = 168.
    ylower = 53.
    yupper = 59.

    # dtopo parameters:
    points_per_degree = 4  # 15 minute resolution
    dx = 1./points_per_degree
    mx = int((xupper - xlower)/dx + 1)
    xupper = xlower + (mx-1)*dx
    my = int((yupper - ylower)/dx + 1)
    yupper = ylower + (my-1)*dx

    x = numpy.linspace(xlower,xupper,mx)
    y = numpy.linspace(ylower,yupper,my)

    times = [1.]
    dtopo = fault.create_dtopography(x,y,times)

    test_data_path = os.path.join(testdir, "data", "sift_test_data.tt3")
    if save:
        dtopo.write(test_data_path, dtopo_type=3)
    compare_data = dtopotools.DTopography(path=test_data_path)
    compare_data.read(path=test_data_path, dtopo_type=3)

    assert dtopo.dZ.shape == compare_data.dZ.shape, \
        "dtopo.dZ.shape is %s, should be %s" \
        % (dtopo.dZ.shape, compare_data.dZ.shape)

    assert numpy.allclose(compare_data.dZ, dtopo.dZ)


def test_SubdividedPlaneFault_make_dtopo(save=False):
    r""""""

    # get a unit source fault plane as starting point:
    sift_slip = {'acsza1':1.}
    fault = dtopotools.SiftFault(sift_slip)
    fault_plane = fault.subfaults[0]
    # Mo = fault_plane.Mo()
    # print "original Mo = ",Mo

    fault2 = dtopotools.SubdividedPlaneFault(fault_plane, nstrike=5, ndip=3)
    # print "new Mo = ",fault2.Mo()
    #fault2.plot_subfaults(slip_color=True)

    assert abs(fault2.Mw() - 7.500686667) < 1e-4, \
           "*** Mw is wrong: %g" % fault.Mw()

    xlower = 162.
    xupper = 168.
    ylower = 53.
    yupper = 59.

    # dtopo parameters:
    points_per_degree = 4  # 15 minute resolution
    dx = 1./points_per_degree
    mx = int((xupper - xlower)/dx + 1)
    xupper = xlower + (mx-1)*dx
    my = int((yupper - ylower)/dx + 1)
    yupper = ylower + (my-1)*dx

    x = numpy.linspace(xlower,xupper,mx)
    y = numpy.linspace(ylower,yupper,my)

    times = [1.]
    dtopo = fault2.create_dtopography(x,y,times)

    test_data_path = os.path.join(testdir, "data", 
            "SubdividedFaultPlane_test_data.tt3")
    if save:
        dtopo.write(test_data_path, dtopo_type=3)
    compare_data = dtopotools.DTopography(path=test_data_path)
    compare_data.read(path=test_data_path, dtopo_type=3)

    assert dtopo.dZ.shape == compare_data.dZ.shape, \
        "dtopo.dZ.shape is %s, should be %s" \
        % (dtopo.dZ.shape, compare_data.dZ.shape)

    assert numpy.allclose(compare_data.dZ, dtopo.dZ)


def test_dtopo_io():
    r"""Test IO of dtopography class"""

    test_data_path = os.path.join(testdir, "data", "alaska1964_test_data.tt3")
    test_dtopo = dtopotools.DTopography(path=test_data_path)

    temp_path = tempfile.mkdtemp()
    try:
        dtopo_paths = [os.path.join(temp_path, 'alaska1964.tt1'),
                       os.path.join(temp_path, 'alaska1964.tt3')]
                       # os.path.join(temp_path, 'alaska1964.tt2'),

        for path in dtopo_paths:
            test_dtopo.write(path)
            dtopo = dtopotools.DTopography(path=path)

            assert test_dtopo.dZ.shape == dtopo.dZ.shape, \
                   "Shape of dZ not equal for topo_type = %s." % dtopo.topo_type

            assert numpy.allclose(test_dtopo.dZ, dtopo.dZ), \
                "dZ not equal for %s" % path

    except AssertionError as e:
        test_dump_path = os.path.join(os.getcwd(), "test_dtopo_io")
        shutil.mkdir(test_dump_path)
        shutil.copy(temp_path, test_dump_path)
        raise e
    finally:
        shutil.rmtree(temp_path)


def test_geometry():
    r"""Test subfault geometry calculation."""

    from . import old_dtopotools

    subfault_path = os.path.join(testdir, 'data', 'alaska1964.csv')
    input_units = {"length":"km", "width":"km", "depth":"km", "slip":"m", 
         "mu":"dyne/cm^2"}
    specifications = ['top center', 'centroid', 'bottom center', 'noaa sift']

    for specification in specifications:
        fault = dtopotools.CSVFault()
        fault.read(subfault_path, input_units=input_units, 
                                  coordinate_specification=specification)

        # Subfault 10 is chosen at random, maybe do all?
        subfault = fault.subfaults[10]
        geometry = old_dtopotools.set_geometry(subfault)

        coord_tests = {"top center":{'test':[geometry['x_top'], 
                                             geometry['y_top'], 
                                             geometry['depth_top']], 
                                 'computed':subfault.centers[0]},
                       "centroid":{'test':[geometry['x_centroid'],
                                           geometry['y_centroid']],
                               'computed':subfault.centers[1][:2]},
                       "bottom center":{"test":[geometry['x_bottom'],
                                                geometry['y_bottom'],
                                                geometry['depth_bottom']],
                                    "computed":subfault.centers[2]},
                       "Corner A":{"test":[geometry["x_corners"][2],
                                           geometry["y_corners"][2]],
                               "computed":subfault.corners[0][:2]},
                       "Corner B":{"test":[geometry["x_corners"][3],
                                           geometry["y_corners"][3]],
                               "computed":subfault.corners[1][:2]},
                       "Corner C":{"test":[geometry["x_corners"][0],
                                           geometry["y_corners"][0]],
                               "computed":subfault.corners[2][:2]},
                       "Corner D":{"test":[geometry["x_corners"][1],
                                           geometry["y_corners"][1]],
                               "computed":subfault.corners[3][:2]}

                    }

        for (values, coord_test) in coord_tests.items():
            assert numpy.allclose(coord_test['test'], coord_test['computed']), \
                   "Specification = %s, coords= %s:\n%s !=\n%s" % (
                                                         specification, 
                                                         values, 
                                                         coord_test['test'], 
                                                         coord_test['computed'])


def test_vs_old_dtopo():
    r"""Test new dtopotools with old version from 5.2"""

    raise nose.SkipTest("Skipping comparison with old tools.")

    from . import old_dtopotools

    temp_path = tempfile.mkdtemp()
    try:
        subfault_path = os.path.join(testdir, 'data', 'alaska1964.csv')
        old_dtopo_file = os.path.join(temp_path, 'old_alaska.tt1')
        old_dtopotools.builddynamicdeffile(subfault_path, subfault_path, 
                                           old_dtopo_file)
    
        input_units = {"length":"km", "width":"km", "depth":"km", "slip":"m", 
                       "mu":"dyne/cm^2"}
        fault = dtopotools.CSVFault()
        fault.read(subfault_path, input_units=input_units, 
                                  coordinate_specification="noaa sift")
        new_dtopo = fault.create_dtopography()

        X, Y, dZ = old_dtopotools.read_dtopo_old(old_dtopo_file, 
                                                 deftype='dynamic', 
                                                 only_last=False)

        assert numpy.allclose(X, new_dtopo.X), \
               "X values do not agree between old and new dtopotools."

        assert numpy.allclose(Y, new_dtopo.Y), \
               "Y values do not agree between old and new dtopotools."

        assert numpy.allclose(dZ, new_dtopo.dZ), \
               "dZ values do not agree between old and new dtopotools."

    except AssertionError as e:
        shutil.copy(temp_path, )
        raise e
    finally:
        shutil.rmtree(temp_path)


def test_dynamic_tohoku(verbose=False, plot=False):
    r"""Test dynamic faulting via a Tohoku example"""

    shoreline_fname = os.path.join(testdir, 'data', 'tohoku_shoreline_1min.npy')
    shoreline_xy = numpy.load(shoreline_fname)

    subfault_fname = os.path.join(testdir, 'data', 'tohoku_ucsb.txt')
    fault = dtopotools.UCSBFault()
    fault.read(subfault_fname)
    fault.rupture_type = 'dynamic'

    if plot:
        import matplotlib.pyplot as plt

        fault.plot_subfaults(slip_color=True)  # plot final slip
        plt.show()

    # seafloor deformation:
    quick_test = True

    if quick_test:
        xlower = 140.
        xupper = 146.
        ylower = 35.
        yupper = 41.
        xylim = [xlower,xupper,ylower,yupper]

        # dtopo parameters for 4 min resolution:
        mx = int((xupper - xlower)*15 + 1)
        my = int((yupper - ylower)*15 + 1)
    else:
        xlower = 135.
        xupper = 150.
        ylower = 30.
        yupper = 45.
        xylim = [xlower,xupper,ylower,yupper]

        # dtopo parameters for 1 min resolution:
        mx = int((xupper - xlower)*60 + 1)
        my = int((yupper - ylower)*60 + 1)


    x = numpy.linspace(xlower,xupper,mx)
    y = numpy.linspace(ylower,yupper,my)

    tmax = 0.
    for s in fault.subfaults:
        tmax = max(tmax, s.rupture_time + s.rise_time + s.rise_time_ending)
    if verbose:
        print("rupture ends at time ",tmax)

    times = numpy.linspace(0,tmax,10)
    dtopo = fault.create_dtopography(x,y,times,verbose=True)

    dz_final = dtopo.dZ[-1]
    dz_max = dz_final.max()

    if plot:
        # Incorporate this function in dtopotools to replace animate_dz_colors?
        def plot_subfaults_dz(t, fig=None):
            if fig is None:
                fig = plt.figure(figsize=(12,5))
            else:
                fig.clf()
            ax1 = fig.add_subplot(121)
            ax2 = fig.add_subplot(122)
            fault.plot_subfaults(axes=ax1, slip_color=True, slip_time=t, xylim=xylim)
            dtopo.plot_dz_colors(axes=ax2, t=t, cmax_dz=dz_max)
            ax1.plot(shoreline_xy[:,0],shoreline_xy[:,1],'g')
            ax2.plot(shoreline_xy[:,0],shoreline_xy[:,1],'g')
            plt.axis(xylim)
            fig.show()
            return fig

        dtopo.plot_dz_colors(t=tmax)
        plt.show()
            
        fig = plt.figure(figsize=(12,5))

        for t in list(numpy.linspace(0,150,16)) + [170,200]:
            plot_subfaults_dz(t,fig)
            plt.draw()
            plt.show()
            time.sleep(1)

    temp_path = tempfile.mkdtemp()
    try:
        fname = os.path.join(temp_path, 'tohoku_ucsb_dynamic.tt3')
        dtopo.write(fname, 3)
    except Exception as e:
        test_name = inspect.stack()[1][-2][0][:-3]
        test_dump_path = os.path.join(os.getcwd(), test_name)
        shutil.mkdir(test_dump_path)
        shutil.copy(temp_path, test_dump_path)
        raise e
    finally:
        shutil.rmtree(temp_path)

    if verbose:
        print('Created ',fname)


def test_subdivided_plane_fault(verbose=False, plot=False):
    r"""Test SubdividedPlaneFault class"""

    # get a unit source fault plane as starting point:
    sift_slip = {'acsza1':1.}
    fault = dtopotools.SiftFault(sift_slip)
    fault_plane = fault.subfaults[0]
    Mo = fault_plane.Mo()
    if verbose:
        print("original Mo = ",Mo)

    fault2 = dtopotools.SubdividedPlaneFault(fault_plane, nstrike=5, ndip=3)
    if verbose:
        print("new Mo = ",fault2.Mo())
    if plot:
        import matplotlib.pyplot as plt
        fault2.plot_subfaults(slip_color=True)
        plt.show()

    slip_function = lambda xi,eta: xi*(1-xi)*eta
    fault2 = dtopotools.SubdividedPlaneFault(fault_plane, nstrike=5, ndip=3, 
            slip_function=slip_function, Mo=Mo)
    if verbose:
        print("new Mo = ",fault2.Mo())
    if plot:
        fault2.plot_subfaults(slip_color=True)
        plt.show()

    fault2.subdivide(nstrike=20, ndip = 10, slip_function=slip_function, Mo=Mo)
    if verbose:
        print("with finer resolution, Mo = ",fault2.Mo())
    if plot:
        fault2.plot_subfaults(slip_color=True)
        plt.show()


if __name__ == "__main__":

    save = False  # default 

    if len(sys.argv) > 1:
        if "plot" in sys.argv[1].lower():
            pass
        elif bool(sys.argv[1]):
            save = True
    try:
        test_read_csv_make_dtopo(save=save)
        test_read_ucsb_make_dtopo(save=save)
        test_read_sift_make_dtopo(save=save)
        test_SubdividedPlaneFault_make_dtopo(save=save)
        test_dtopo_io()
        test_geometry()
        test_vs_old_dtopo()
    except nose.SkipTest as e:
        print(e.message)



#!/usr/bin/env python
# coding: utf-8

# # Make input files for geoclaw_fgmax_test
# 
# Create synthetic topo files and fgmax input file fgmax_pts_island.data specified as a topo_type==3 file.

# In[ ]:


from pylab import *
import os,sys
from clawpack.visclaw import colormaps, plottools
from clawpack.geoclaw import fgmax_tools, topotools
from clawpack.visclaw.plottools import pcolorcells


# In[ ]:


zmin = -60.
zmax = 40.

land_cmap = colormaps.make_colormap({ 0.0:[0.1,0.4,0.0],
                                     0.25:[0.0,1.0,0.0],
                                      0.5:[0.8,1.0,0.5],
                                      1.0:[0.8,0.5,0.2]})

sea_cmap = colormaps.make_colormap({ 0.0:[0,0,1], 1.:[.8,.8,1]})

cmap, norm = colormaps.add_colormaps((land_cmap, sea_cmap),
                                     data_limits=(zmin,zmax),
                                     data_break=0.)
                                     
sea_cmap_dry = colormaps.make_colormap({ 0.0:[1.0,0.7,0.7], 1.:[1.0,0.7,0.7]})
cmap_dry, norm_dry = colormaps.add_colormaps((land_cmap, sea_cmap_dry),
                                     data_limits=(zmin,zmax),
                                     data_break=0.)


# ## Run script to generate topo files

# In[ ]:


import maketopo
maketopo.maketopo()
maketopo.makeqinit()


# In[ ]:


ocean_topo = topotools.Topography(path='ocean.tt3', topo_type=3)
island_topo = topotools.Topography(path='island.tt3', topo_type=3)
ocean_topo.plot()
plottools.plotbox(island_topo.extent, kwargs={'color':'r'})
title('Ocean topo\nRed box is location of island');


# In[ ]:


island_topo.plot()
title('Island topo')


# In[ ]:


extent = [14.25, 14.65, 50.1, 50.35]
topo = island_topo.crop(extent)

figure(figsize=(12,6))
pcolorcells(topo.X, topo.Y, topo.Z, cmap=cmap, norm=norm)
colorbar(extend='both', ticks=arange(zmin,zmax+1,10))
gca().set_aspect(1./cos(50.2*pi/180.))


# ## Select only points in this region near shore 
# 
# Select the points where `Z > -15`m or `Z < 15`m elevation.

# In[ ]:


pts_chosen = where(logical_and(topo.Z>-15, topo.Z<15), 1, 0)


# In[ ]:


Zchosen = ma.masked_array(topo.Z, logical_not(pts_chosen))


# In[ ]:


figure(figsize=(12,6))
pcolorcells(topo.X, topo.Y, Zchosen, cmap=cmap, norm=norm)
colorbar(extend='both', ticks=arange(zmin,zmax+1,10))
gca().set_aspect(1./cos(48*pi/180.))


# ### Write file indicating these points
# 
# Create a file that is like a topo DEM file with `topo_type==3` but with `Z` values equal to either 1 (if point is to be used as an fgmax point) or 0 (if not).
# 
# This file can be used together with setting `point_style==4` in `setrun.py`.

# In[ ]:


fname_fgmax_mask = 'fgmax_pts_island.data'
topo_fgmax_mask = topotools.Topography()

#topo_fgmax_mask.set_xyZ(topo.x, topo.y, chosen) # new in 5.7.0 will replace lines below
topo_fgmax_mask._x = topo.x
topo_fgmax_mask._y = topo.y
topo_fgmax_mask._Z = pts_chosen
topo_fgmax_mask.generate_2d_coordinates()

topo_fgmax_mask.write(fname_fgmax_mask, topo_type=3, Z_format='%1i')
print('Created %s' % fname_fgmax_mask)


# In[ ]:







#!/usr/bin/env python

import os
import sys
import tempfile
import shutil

try:
    # For Python 3.0 and later
    from urllib.error import URLError
except ImportError:
    # Fall back to Python 2's urllib2
    from urllib2 import URLError

import numpy

import nose

import clawpack.geoclaw.topotools as topotools
import clawpack.clawutil.data

# Set local test directory to get local files
testdir = os.path.dirname(__file__)
if len(testdir) == 0:
     testdir = "./"


def topo_bowl(x,y):
    """Sample topo"""
    z = 1000.*(x**2 + y**2 - 1.)
    return z


def test_read_write_topo_bowl():
    """
    Test writing and reading topo files with small number of points
    Note that ordering should go from NW corner.
    """

    # Base topography
    topo = topotools.Topography(topo_func=topo_bowl)
    topo.x = numpy.linspace(-1.0, 3.0, 5)
    topo.y = numpy.linspace( 0.0, 3.0, 4)

    assert numpy.allclose(topo.x, numpy.array([-1.,  0.,  1.,  2., 3.])), \
           "Topography x values are incorrect."
    assert numpy.allclose(topo.X,
                          numpy.array([[-1.,  0.,  1.,  2.,  3.],
                                       [-1.,  0.,  1.,  2.,  3.],
                                       [-1.,  0.,  1.,  2.,  3.],
                                       [-1.,  0.,  1.,  2.,  3.]])), \
           "Topography X values are incorrect."
    assert numpy.allclose(topo.y, numpy.array([ 0.,  1.,  2.,  3.])), \
           "Topography y values are incorrect."
    assert numpy.allclose(topo.Y,
                          numpy.array([[ 0.,  0.,  0.,  0.,  0.],
                                       [ 1.,  1.,  1.,  1.,  1.],
                                       [ 2.,  2.,  2.,  2.,  2.],
                                       [ 3.,  3.,  3.,  3.,  3.]])), \
           "Topography Y values are incorrect."
    assert numpy.allclose(topo.Z,
                numpy.array([[     0.,  -1000.,      0.,   3000., 8000.],
                             [  1000.,      0.,   1000.,   4000.,   9000.],
                             [  4000.,   3000.,   4000.,   7000.,  12000.],
                             [  9000.,   8000.,   9000.,  12000.,  17000.]])), \
           "Topography Z values are incorrect."

    temp_path = tempfile.mkdtemp()
    try:
        for topo_type in range(1, 4):
            path = os.path.join(temp_path, 'bowl.tt%s' % topo_type)
            topo.write(path, topo_type=topo_type,Z_format="%22.15e")

            topo_in = topotools.Topography(path)
            assert numpy.allclose(topo.Z, topo_in.Z), \
                   "Differnece in written and read topography found."
    except AssertionError as e:
        # If the assertion failed then copy the contents of the directory
        shutil.copytree(temp_path, os.path.join(os.getcwd(), 
                                                "test_read_write_topo_bowl"))
        raise e
    finally:
        shutil.rmtree(temp_path)


def test_crop_topo_bowl():
    """
    Test cropping a topo file.
    """

    topo = topotools.Topography(topo_func=topo_bowl)
    topo.x = numpy.linspace(-1.0, 3.0, 5)
    topo.y = numpy.linspace( 0.0, 3.0, 4)

    # topo.Z should be created automatically when referenced below:
    assert numpy.allclose(topo.Z,
           numpy.array([[     0.,  -1000.,      0.,   3000., 8000.],
                        [  1000.,      0.,   1000.,   4000.,   9000.],
                        [  4000.,   3000.,   4000.,   7000.,  12000.],
                        [  9000.,   8000.,   9000.,  12000.,  17000.]])), \
           "Basic topography does not match test data."

    cropped_topo = topo.crop([0, 1, 0, 2])
    assert numpy.allclose(cropped_topo.x, numpy.array([0.0, 1.0])), \
           "Cropped topography y values do not match"
    assert numpy.allclose(cropped_topo.y, numpy.array([ 0.,  1.,  2.])), \
           "Cropped topography y values do not match."
    assert numpy.allclose(cropped_topo.Z,
                          numpy.array([[-1000.,     0.],
                                       [    0.,  1000.],
                                       [ 3000.,  4000.]])), \
           "Cropped topography Z values do not match."



def test_against_old():
    """
    Test against the old topotools from 5.1.0.
    Compare bowl.tt1 to bowl_old.tt1
    """
    
    from . import old_topotools

    nxpoints = 5
    nypoints = 4
    xlower = -1.0
    xupper = 3.0
    ylower = 0.0
    yupper = 3.0
    topo = topotools.Topography(topo_func=topo_bowl)
    topo.x = numpy.linspace(xlower, xupper, nxpoints)
    topo.y = numpy.linspace(ylower, yupper, nypoints)

    temp_path = tempfile.mkdtemp()
    try:
        file_path = os.path.join(temp_path, "bowl_old.tt1")
        old_topotools.topo1writer(file_path, topo_bowl, xlower, xupper, ylower, 
                                  yupper, nxpoints, nypoints)
        X, Y, Z = old_topotools.topofile2griddata(file_path, topotype=1)
        Y = numpy.flipud(Y)
        Z = numpy.flipud(Z)

        assert numpy.allclose(topo.X, X), "Difference in X grid."
        assert numpy.allclose(topo.Y, Y), "Difference in Y grid."
        assert numpy.allclose(topo.Z, Z), "Difference in Z grid."

    except AssertionError as e:
        # If the assertion failed then copy the contents of the directory
        shutil.copytree(temp_path, os.path.join(os.getcwd(), 
                                                "test_against_old"))
        raise e
    finally:
        shutil.rmtree(temp_path)



def topo_bowl_hill(x,y):
    """
    Sample topography
    """
    # Parabolic bowl
    z = 1000.*(x**2 + y**2 - 1.)
    # Add a Gaussian hill
    z = z + 1000.*numpy.exp(-100*((x-0.7)**2 + (y-0.8)**2))
    return z


def test_read_write_topo_bowl_hill():
    """
    Test writing and reading topo files.
    """
    temp_path = tempfile.mkdtemp()

    try:
        topo = topotools.Topography(topo_func=topo_bowl_hill)
        topo.x = numpy.linspace(-1.5, 2.5, 101)
        topo.y = numpy.linspace(-1.0, 2.0, 76)

        for topo_type in range(1,4):
            file_path = os.path.join(temp_path, 'bowl_hill.tt%s' % topo_type)
            topo.write(file_path, topo_type=topo_type,Z_format="%22.15e")
            topo_in = topotools.Topography(path=file_path, topo_type=topo_type)
            assert numpy.allclose(topo.Z, topo_in.Z), \
                   "Written file of topo_type=%s does not equal read in" + \
                   " file." % topo_type

    except AssertionError as e:
        # If the assertion failed then copy the contents of the directory
        shutil.copytree(temp_path, os.path.join(os.getcwd(), 
                                              "test_read_write_topo_bowl_hill"))
        raise e
    finally:
        shutil.rmtree(temp_path)


def test_netcdf():
    r"""Test Python NetCDF formatted topography reading"""

    temp_path = tempfile.mkdtemp()

    try:
        # Fetch comparison data
        url = "".join(('https://raw.githubusercontent.com/rjleveque/geoclaw/',
                       '5f675256c043e59e5065f9f3b5bdd41c2901702c/src/python/',
                       'geoclaw/tests/kahului_sample_1s.tt2'))
        clawpack.clawutil.data.get_remote_file(url, output_dir=temp_path,
                                                    force=True)
        
        # Paths
        local_path = os.path.join(temp_path, os.path.basename(url))
        nc_path = os.path.join(temp_path, "test.nc")

        # Write out NetCDF version of file
        ascii_topo = topotools.Topography(path=local_path)
        ascii_topo.read()
        ascii_topo.write(nc_path, topo_type=4,Z_format="%22.15e")

        # Read back in NetCDF file
        nc_topo = topotools.Topography(path=nc_path)
        nc_topo.read()

        # Compare arrays - use tolerance based on 30 arcsecond accuracy
        assert numpy.allclose(ascii_topo.x, nc_topo.x), \
                    "Flat x-arrays did not match."
        assert numpy.allclose(ascii_topo.y, nc_topo.y), \
                    "Flat y-arrays did not match."
        assert numpy.allclose(ascii_topo.Z, nc_topo.Z), \
                    "Flat y-arrays did not match."

    except AssertionError as e:
        shutil.copytree(temp_path, os.path.join(os.getcwd()),
            'test_read_netcdf')
        raise e

    except ImportError as e:
        raise nose.SkipTest("Skipping test since NetCDF support not found.")

    except RuntimeError as e:
        raise nose.SkipTest("NetCDF topography test skipped due to " +
                            "runtime failure.")
    except URLError:
        raise nose.SkipTest("Could not fetch remote file, skipping test.")
    
    finally:
        shutil.rmtree(temp_path)


def test_get_remote_file():
    """Test the ability to fetch a remote file from the web."""
    
    temp_path = tempfile.mkdtemp()
    try:

        url = "".join(('https://raw.githubusercontent.com/rjleveque/geoclaw/',
                       '5f675256c043e59e5065f9f3b5bdd41c2901702c/src/python/',
                       'geoclaw/tests/kahului_sample_1s.tt2'))
        clawpack.clawutil.data.get_remote_file(url, output_dir=temp_path,
            force=True)

        local_path = os.path.join(temp_path, os.path.basename(url))
        download_topo = topotools.Topography(path=local_path)

        test_path = os.path.join(testdir, "data", os.path.basename(url))
        test_topo = topotools.Topography(path=test_path)

        assert numpy.allclose(download_topo.Z, test_topo.Z), \
               "Downloaded file does not match %s" % test_path
    except AssertionError as e:
        shutil.copy(local_path, os.path.join(os.getcwd(), "remote_file.tt2"))
        raise e

    except URLError:
        raise nose.SkipTest("Could not fetch remote file, skipping test.")

    finally:
        shutil.rmtree(temp_path)


def test_unstructured_topo(save=False, plot=False):

    try:
        import scipy
    except:
        raise nose.SkipTest("Skipping test since scipy not found")

    # Create random test data
    def func(x, y):
        return x * (1 - x) * numpy.cos(4 * numpy.pi * x) * numpy.sin(4 * numpy.pi * y**2)**2

    fill_topo = topotools.Topography()
    fill_topo.x = numpy.linspace(0, 1, 100)
    fill_topo.y = numpy.linspace(0, 1, 200)
    fill_topo.Z = func(fill_topo.X, fill_topo.Y)

    points = numpy.loadtxt(os.path.join(testdir, "data", 
                                                 "unstructured_points.txt"))
    values = func(points[:,0], points[:,1])

    # Create topography object
    topo = topotools.Topography(unstructured=True)
    topo.x = points[:,0]
    topo.y = points[:,1]
    topo.z = values

    if plot:
        import matplotlib.pyplot as plt

        fig = plt.figure(figsize=(16,6))
        axes = fig.add_subplot(1, 3, 1)
        fill_topo.plot(axes=axes)
        axes.set_title("True Field")
        axes = fig.add_subplot(1, 3, 2)
        topo.plot(axes=axes, region_extent=[0, 1, 0, 1])
        axes.set_title("Unstructured Field")

    topo.interp_unstructured(fill_topo, extent=[0, 1, 0, 1], delta=(1e-2,1e-2))
    assert not topo.unstructured

    # Load (and save) test data and make the comparison
    test_data_path = os.path.join(testdir, "data", "unstructured_test_data.tt3")
    if save:
        topo.write(test_data_path,Z_format="%22.15e")

    compare_data = topotools.Topography(path=test_data_path)

    assert numpy.allclose(compare_data.Z, topo.Z)

    if plot:
        axes = fig.add_subplot(1, 3, 3)
        topo.plot(axes=axes)
        axes.set_title("Interpolated Field")

        plt.show()


def plot_topo_bowl_hill():

    """
    Create topo and write out, then read in again and plot.
    Note that center of bowl should be at (0,0).
    """

    try:
        import matplotlib
    except ImportError:
        raise nose.SkipTest("Skipping test since matplotlib not found.")

    matplotlib.use("Agg")  # use image backend -- needed for Travis tests
    import matplotlib.pyplot as plt

    topo = topotools.Topography(topo_func=topo_bowl_hill)
    topo.x = numpy.linspace(-1.5, 2.5, 101)
    topo.y = numpy.linspace(-1.0, 2.0, 76)
    fname = 'bowl_hill.tt2'
    topo = topotools.Topography(fname,topo_type=2)

    topo.plot()
    fname = "bowl_hill.png"
    plt.savefig(fname)
    print("Created ",fname)

    topo2 = topo.crop([0.5, 1.5, 0., 2.])
    topo2.plot()
    plt.title("Cropped topography")
    fname = "bowl_hill_crop.png"
    plt.savefig(fname)
    print("Created ",fname)


def plot_kahului():
    r"""
    Example illustrating reading in a topo file and plotting.
    Uses the test data kahului_sample_1s.tt2, created by cropping 
    the data file obtained from the NGDC site
        http://www.ngdc.noaa.gov/dem/squareCellGrid/download/604
    In addition to using the Topography.plot function, also 
    illustrate how to do a contour data of the data directly.
    """

    try:
        import matplotlib
    except ImportError:
        raise nose.SkipTest("Skipping test since matplotlib not found.")

    matplotlib.use("Agg")  # use image backend -- needed for Travis tests
    import matplotlib.pyplot as plt

    path = os.path.join(testdir,'kahului_sample_1s.tt2')
    K = topotools.Topography(path,topo_type=2)
    K.plot()
    plt.title("Kahului Harbor at 1 second resolution")

    plt.title("Kahului Harbor at 1 second resolution")
    fname = "kahului_imshow.png"
    plt.savefig(fname)
    print("Created ",fname)

    assert K.Z.shape == (46, 65), "*** K.Z is wrong shape"
    assert numpy.allclose(K.Z[:3,:3], \
                          numpy.array([[ 11.339,  11.339, 11.339],
                                       [ 13.339,  11.339,  11.339],
                                       [ 13.339,  11.339, 10.339]])), \
                "*** Topography K does not match"

    # Make a contour plot of topography / bathymetry:
    plt.figure()
    ax = plt.axes()
    plt.contour(K.X, K.Y, K.Z, numpy.linspace(-20,-2,10), colors='b', \
                linestyles='-')
    plt.contour(K.X, K.Y, K.Z, numpy.linspace(2,20,10), colors='g')
    plt.contour(K.X, K.Y, K.Z, [0.], colors='r')  # mean high water

    # fix aspect ratio based on latitude:
    mean_lat = 0.5 * (K.y.max() + K.y.min())
    ax.set_aspect(1.0 / numpy.cos(numpy.pi / 180.0 * mean_lat))

    # fix tick marks so readable:
    ax.ticklabel_format(format="plain", useOffset=False)
    plt.xticks(rotation=20)

    plt.title("2-meter contours of topo (green) and bathymetry (blue)",\
              fontsize=12)
    fname = "kahului_contour.png"
    plt.savefig(fname)
    print("Created ",fname)


if __name__ == "__main__":
    if len(sys.argv) > 1:
        if "plot" in sys.argv[1].lower():
            plot_kahului()
            plot_topo_bowl_hill()
            test_unstructured_topo(save=False, plot=True)
        elif bool(sys.argv[1]):
            test_unstructured_topo(save=True)
    else:
        # Run tests one at a time
        test_read_write_topo_bowl()
        test_crop_topo_bowl()
        test_against_old()
        test_read_write_topo_bowl_hill()
        test_get_remote_file()
        test_unstructured_topo()
        test_netcdf()

        print("All tests passed.")



num_eqn = {
        'acoustics_1D' : 2,
        'acoustics_variable_1D' : 2,
        'advection_1D' : 1,
        'advection_color_1D' : 1,
        'advection_1D_py' : 1,
        'burgers_1D' : 1,
        'burgers_1D_py' : 1,
        'cubic_1D' : 1,
        'traffic_1D' : 1,
        'traffic_vc_1D' : 1,
        'traffic_vc_fwave_1D' : 1,
        'traffic_vc_tracer_1D' : 2,
        'euler_with_efix_1D' : 3,
        'euler_roe_1D' : 3,
        'euler_hll_1D' : 3,
        'euler_hllc_1D' : 3,
        'euler_hlle_1D' : 3,
        'mhd_roe_1D' : 8,
        'nonlinear_elasticity_fwave_1D' : 2,
        'psystem_fwave_1D' : 2,
        'nonlinear_elasticity_1D' : 2,
        'reactive_euler_with_efix_1D' : 4,
        'shallow_roe_with_efix_1D' : 2,
        'shallow_roe_tracer_1D' : 3,
        'shallow_roe_1D' : 2,
        'shallow_hll_1D' : 2,
        'shallow_hlle_1D' : 2,
        'shallow_bathymetry_fwave_1D' : 2,
        'shallow_1D_py' : 2,
        'vc_advection_1D' : 1,
        'layered_shallow_water_1D' : 4,
        'acoustics_2D' : 3,
        'acoustics_mapped_2D' : 3,
        'advection_2D' : 1,
        'burgers_2D' : 1,
        'euler_mapgrid_2D' : 4,
        'euler_5wave_2D' : 5,
        'euler_4wave_2D' : 4,
        'euler_hlle_2D' : 4,
        'euler_hlle_with_walls_2D' : 4,
        'kpp_2D' : 1,
        'psystem_2D' : 3,
        'shallow_hlle_2D' : 3,
        'shallow_roe_with_efix_2D' : 3,
        'shallow_bathymetry_fwave_2D' : 3,
        'sw_aug_2D' : 3,
        'shallow_sphere_2D' : 4,
        'vc_advection_2D' : 1,
        'vc_acoustics_2D' : 3,
        'vc_elasticity_2D' : 5,
        'vc_acoustics_3D' : 4,
        'euler_3D' : 5
        }

num_waves = {
        'acoustics_1D' : 2,
        'acoustics_variable_1D' : 2,
        'advection_1D' : 1,
        'advection_color_1D' : 1,
        'advection_1D_py' : 1,
        'burgers_1D' : 1,
        'burgers_1D_py' : 1,
        'cubic_1D' : 1,
        'traffic_1D' : 1,
        'traffic_vc_1D' : 1,
        'traffic_vc_fwave_1D' : 1,
        'traffic_vc_tracer_1D' : 2,
        'euler_with_efix_1D' : 3,
        'euler_roe_1D' : 3,
        'euler_hll_1D' : 2,
        'euler_hllc_1D' : 3,
        'euler_hlle_1D' : 2,
        'mhd_roe_1D' : 7,
        'nonlinear_elasticity_fwave_1D' : 2,
        'nonlinear_elasticity_1D' : 2,
        'psystem_fwave_1D' : 2,
        'reactive_euler_with_efix_1D' : 4,
        'shallow_roe_with_efix_1D' : 2,
        'shallow_roe_tracer_1D' : 3,
        'shallow_roe_1D' : 2,
        'shallow_hll_1D' : 2,
        'shallow_hlle_1D' : 2,
        'shallow_bathymetry_fwave_1D' : 2,
        'shallow_1D_py' : 2,
        'vc_advection_1D' : 1,
        'layered_shallow_water_1D' : 4,
        'acoustics_2D' : 2,
        'acoustics_mapped_2D' : 2,
        'advection_2D' : 1,
        'burgers_2D' : 1,
        'euler_mapgrid_2D' : 4,
        'euler_5wave_2D' : 5,
        'euler_4wave_2D' : 4,
        'euler_hlle_2D' : 2,
        'euler_hlle_with_walls_2D' : 2,
        'kpp_2D' : 1,
        'psystem_2D' : 2,
        'shallow_hlle_2D' : 2,
        'shallow_roe_with_efix_2D' : 3,
        'shallow_bathymetry_fwave_2D' : 3,
        'sw_aug_2D' : 3,
        'shallow_sphere_2D' : 3,
        'vc_advection_2D' : 1,
        'vc_acoustics_2D' : 2,
        'vc_elasticity_2D' : 4,
        'vc_acoustics_3D' : 2,
        'euler_3D' : 3
        }
